@@ -1,6 +1,6 @@
 package MooseX::StrictConstructor::Trait::Class;
-BEGIN {
-  $MooseX::StrictConstructor::Trait::Class::VERSION = '0.15';
+{
+  $MooseX::StrictConstructor::Trait::Class::VERSION = '0.17';
 }
 
 use Moose::Role;
@@ -9,7 +9,33 @@ use namespace::autoclean;
 
 use B ();
 
-around '_inline_BUILDALL' => sub {
+around new_object => sub {
+    my $orig     = shift;
+    my $self     = shift;
+    my $params   = @_ == 1 ? $_[0] : {@_};
+    my $instance = $self->$orig(@_);
+
+    my %attrs = (
+        __INSTANCE__ => 1,
+        (
+            map { $_ => 1 }
+            grep {defined}
+            map  { $_->init_arg() } $self->get_all_attributes()
+        )
+    );
+
+    my @bad = sort grep { !$attrs{$_} } keys %$params;
+
+    if (@bad) {
+        $self->throw_error(
+            "Found unknown attribute(s) init_arg passed to the constructor: @bad"
+        );
+    }
+
+    return $instance;
+};
+
+around _inline_BUILDALL => sub {
     my $orig = shift;
     my $self = shift;
 
@@ -24,27 +50,13 @@ around '_inline_BUILDALL' => sub {
 
     return (
         @source,
-        'my %attrs = (' . ( join ' ', @attrs ) . ');',
-        'my @bad = sort grep { !$attrs{$_} } keys %{ $params };',
+        'my @bad = sort grep { !$allowed_attrs{$_} } keys %{ $params };',
         'if (@bad) {',
             'Moose->throw_error("Found unknown attribute(s) passed to the constructor: @bad");',
         '}',
     );
 } if $Moose::VERSION >= 1.9900;
 
-# If the base class role is applied first, and then a superclass is added, we
-# lose the role.
-after superclasses => sub {
-    my $self = shift;
-
-    return unless @_;
-
-    Moose::Util::MetaRole::apply_base_class_roles(
-        for   => $self->name(),
-        roles => ['MooseX::StrictConstructor::Role::Object'],
-    );
-};
-
 1;
 
 # ABSTRACT: A role to make immutable constructors strict
@@ -59,7 +71,7 @@ MooseX::StrictConstructor::Trait::Class - A role to make immutable constructors
 
 =head1 VERSION
 
-version 0.15
+version 0.17
 
 =head1 DESCRIPTION
 
@@ -73,7 +85,7 @@ Dave Rolsky <autarch@urth.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2011 by Dave Rolsky.
+This software is Copyright (c) 2012 by Dave Rolsky.
 
 This is free software, licensed under:
 
